package org.openxma.dsl.pom.naming;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.naming.QualifiedName;
import org.openxma.dsl.pom.formatting.PomModelSerializer;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.TableColumn;

/**
 * This class is intended to be used for providing composed names which
 * are derived from members of a ModelElement which are references. The 
 * composition of such a name causes sometimes a recursive proxy 
 * resolving which throws an Exception.
 * 
 *  TODO Improve this (temporary) solution.
 *
 */
public class CustomPomDslNameProvider extends PomNameProvider {
	
	/**
	 * Constructor
	 */
	public CustomPomDslNameProvider() {
		super();
	}
	
	/**
	 * Get and/or compose the qualified name
	 */
	public QualifiedName getFullyQualifiedName(EObject from) {
		if (from instanceof CustomizeableField) {
			CustomizeableField customizeableField = (CustomizeableField) from;
			StringBuilder buf = new StringBuilder();
			buf.append(customizeableField.getObject().getName());
			buf.append(PomModelSerializer.getFeatureDefinitionString(customizeableField.getFeature()));
			return createQualifiedName(buf.toString());					
		} if (from instanceof TableColumn) {
		    TableColumn tableColumn = (TableColumn)from;
		    if (tableColumn.getColumnName() != null) {
		        return QualifiedName.create(tableColumn.getColumnName());
		    }
		    String featureDefinitionString = PomModelSerializer.getFeatureDefinitionString(tableColumn.getFeature());		    
            StringBuilder buf = new StringBuilder();
            if (tableColumn.getObject() != null) {
                buf.append(tableColumn.getObject().getName());
            } else {
                if (featureDefinitionString.startsWith(".")) {
                    featureDefinitionString=featureDefinitionString.substring(1);
                }
            }            
            buf.append(featureDefinitionString);
            return createQualifiedName(buf.toString());		    		
		} else {
			String name = super.getResolver(from).apply(from);
            return name!=null ? QualifiedName.create(name) : null;
		}
	}

    private QualifiedName createQualifiedName(String name) {
        return QualifiedName.create(name.split("\\."));
    }
}
